﻿using Foundation;
using Uno.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UIKit;
using Windows.UI.Core;
using System.Threading.Tasks;

namespace Windows.UI.Xaml.Controls
{
	public partial class SinglelineTextBoxDelegate : UITextFieldDelegate
	{
		private WeakReference<TextBox> _textBox;

		public SinglelineTextBoxDelegate(WeakReference<TextBox> textbox)
		{
			_textBox = textbox;
		}

		public bool IsKeyboardHiddenOnEnter
		{
			get;
			set;
		}

		public override bool ShouldChangeCharacters(UITextField textField, NSRange range, string replacementString)
		{
			var textBoxView = textField as SinglelineTextBoxView;
			if (textBoxView != null)
			{
                if(_textBox.GetTarget()?.OnKey(replacementString.FirstOrDefault()) ?? false)
                {
                    return false;
                }

                if (_textBox.GetTarget()?.MaxLength > 0)
				{
					var newLength = textBoxView.Text.Length + replacementString.Length - range.Length;
					return newLength <= _textBox.GetTarget()?.MaxLength;
				};
			}

			return true;
		}

		public override bool ShouldReturn(UITextField textField)
		{
			if (IsKeyboardHiddenOnEnter)
			{
				CoreDispatcher.Main.RunAsync(CoreDispatcherPriority.Normal,
					async () =>
					{
						// Delay losing focus to avoid concurrent interactions when transferring focus to another control. See 101152
						await Task.Delay(TimeSpan.FromMilliseconds(50));
						textField.ResignFirstResponder();
					});
			}

			var textBox = textField as SinglelineTextBoxView;
			if (textBox != null)
			{
                if (_textBox.GetTarget()?.OnKey('\n') ?? false)
                {
                    return false;
                }
			}

			return true;
		}

		public override bool ShouldBeginEditing(UITextField textField)
		{
			return !_textBox.GetTarget()?.IsReadOnly ?? false;
		}

		/// <summary>
		/// Corresponds to a gain of focus
		/// </summary>
		public override void EditingStarted(UITextField textField)
		{
			// Using FocusState.Pointer by default until need to distinguish between Pointer, Programmatic and Keyboard.
			_textBox.GetTarget()?.Focus(FocusState.Pointer);
		}

		/// <summary>
		/// Corresponds to a loss of focus
		/// </summary>
		public override void EditingEnded(UITextField textField)
		{
			// TODO: Remove when Focus is implemented correctly
			_textBox.GetTarget()?.Unfocus();
		}
	}
}
